<?php

namespace App\Controller;

use App\Entity\Invoice;
use App\Entity\InvoiceLine;
use App\Entity\Purchase;
use App\Form\Invoice1Type;
use App\Helper\InvoiceFactory;
use App\Repository\InvoiceRepository;
use App\Service\PdfService;
use DateTimeImmutable;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Address;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/admin/invoice")
 */
class AdminInvoiceController extends AbstractController
{
    private function handleInvoiceLines(Invoice $invoice, EntityManagerInterface $entityManager) {
        foreach($invoice->getInvoiceLines() as $invoiceLine) {
            if (!$invoiceLine->getDescription()) {
                if ($invoiceLine->getId()) {
                    $entityManager->remove($invoiceLine);
                }
                $invoice->removeInvoiceLine($invoiceLine);
            } else {
                $entityManager->persist($invoiceLine);
            }
        }
    }

    /**
     * @Route("/", name="admin_invoice_index", methods={"GET"})
     */
    public function index(InvoiceRepository $invoiceRepository): Response
    {
        return $this->render('admin_invoice/index.html.twig', [
            'invoices' => $invoiceRepository->findAll(),
        ]);
    }

    /**
     * @Route("/new/{id}", name="admin_invoice_new_from_purchase", methods={"GET","POST"})
     */
    public function newFromPurchase(Request $request, Purchase $purchase): Response
    {
        $invoice = InvoiceFactory::create($purchase);
        return $this->subNew($request, $invoice, $purchase);
    }

    /**
     * @Route("/new", name="admin_invoice_new", methods={"GET","POST"})
     */
    public function new(Request $request): Response
    {
        $invoice = InvoiceFactory::create(null);
        return $this->subNew($request, $invoice, null);
    }

    private function subNew(Request $request, Invoice $invoice, ?Purchase $purchase) {
        $form = $this->createForm(Invoice1Type::class, $invoice);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager = $this->getDoctrine()->getManager();

            $this->handleInvoiceLines($invoice, $entityManager);

            $entityManager->persist($invoice);
            $entityManager->flush();

            return $this->redirectToRoute('admin_invoice_index', [], Response::HTTP_SEE_OTHER);
        }

        return $this->render('admin_invoice/new.html.twig', [
            'invoice' => $invoice,
            'form' => $form->createView(),
            'purchase' => $purchase
        ]);
    }

    /**
     * @Route("/{id}", name="admin_invoice_show", methods={"GET"})
     */
    public function show(Invoice $invoice): Response
    {
        return $this->render('admin_invoice/show.html.twig', [
            'invoice' => $invoice,
        ]);
    }

    /**
     * @Route("/{id}/edit", name="admin_invoice_edit", methods={"GET","POST"})
     */
    public function edit(Request $request, Invoice $invoice): Response
    {
        for ($i = 0; $i < 5; ++$i) {
            $invoice->addInvoiceLine(new InvoiceLine());
        }

        $form = $this->createForm(Invoice1Type::class, $invoice);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager = $this->getDoctrine()->getManager();

            $this->handleInvoiceLines($invoice, $entityManager);

            $entityManager->flush();

            return $this->redirectToRoute('admin_invoice_index', [], Response::HTTP_SEE_OTHER);
        }

        return $this->render('admin_invoice/edit.html.twig', [
            'invoice' => $invoice,
            'form' => $form->createView(),
        ]);
    }

    /**
     * @Route("/{id}/add/lines/{type}", name="admin_invoice_add_lines", methods={"POST"})
     */
    public function addLines(Request $request, Invoice $invoice, string $type): Response
    {
        $form = $this->createForm(Invoice1Type::class, $invoice);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            $entityManager = $this->getDoctrine()->getManager();

            $this->handleInvoiceLines($invoice, $entityManager);

            $entityManager->flush();
        }

        if ($type === 'edit') {
            return $this->redirectToRoute('admin_invoice_edit', [
                'id' => $invoice->getId()
            ]);
        } else
        {
            return $this->redirectToRoute('admin_invoice_new', [
                'id' => $invoice->getId()
            ]);
        }
    }

    /**
     * @Route("/{id}", name="admin_invoice_delete", methods={"POST"})
     */
    public function delete(Request $request, Invoice $invoice): Response
    {
        if ($this->isCsrfTokenValid('delete'.$invoice->getId(), $request->request->get('_token'))) {
            $entityManager = $this->getDoctrine()->getManager();
            $entityManager->remove($invoice);
            $entityManager->flush();
        }

        return $this->redirectToRoute('admin_invoice_index', [], Response::HTTP_SEE_OTHER);
    }

    private function getInvoiceAsHtml(Invoice $invoice): string {
        $priceTotalTtc = 0.0;

        foreach ($invoice->getInvoiceLines() as $line) {
            $priceTotalTtc += $line->getPriceTtc();
        }

        return $this->renderView('admin_invoice/pdf.html.twig', [
            'invoice' => $invoice,
            'priceTotalHt' => $priceTotalTtc,
            'priceTotalTtc' => $priceTotalTtc
        ]);
    }

    /**
     * @Route("/{id}/preview", name="admin_invoice_preview", methods={"GET"})
     */
    public function preview(Invoice $invoice): Response
    {
        return $this->render('admin_invoice/preview.html.twig', [
            'invoice' => $invoice,
            'html' => $this->getInvoiceAsHtml($invoice)
        ]);
    }

    /**
     * @Route("/{id}/send", name="admin_invoice_send", methods={"GET"})
     */
    public function send(Invoice $invoice, EntityManagerInterface $entityManager, PdfService $pdfService, MailerInterface $mailer): Response
    {
        $filename = $invoice->getReference() . '.pdf';
        $outFilePath = $pdfService->generate($this->getInvoiceAsHtml($invoice), $filename);

        $email = (new TemplatedEmail())
            ->from(new Address('no-reply@id-3d.fr', 'No Reply'))
            ->cc('nguyen.hoang.michael@gmail.com')
            ->to($invoice->getPurchase()->getUser()->getEmail())
            ->subject('Votre facture ID-3D')
            ->htmlTemplate('admin_invoice/email.html.twig')
            ->attachFromPath($outFilePath)
        ;
        $mailer->send($email);

        $invoice->setFilename($filename);
        $invoice->setStatus('sent_but_not_paid');
        $entityManager->flush();

        return $this->render('admin_invoice/send.html.twig', [
            'invoice' => $invoice
        ]);
    }
}
